﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DAL;
using Domain;

namespace ConsoleApp
{
    class Program
    {
        static void Main(string[] args)
        {
            var db = new DbContext();
            db.SeedData();

            for (;;)
            {
                OutputMenuMain();
                var command = GetInput().Trim().ToUpper();
                Console.WriteLine("Command: " + command);

                switch (command)
                {
                    case "I":
                        break;
                    case "T":
                        CommandContactType(db);
                        break;
                    case "P":
                        CommandPerson(db);
                        break;
                    case "C":
                        CommandContact(db);
                        break;
                    case "E":
                        break;
                    default:
                        break;
                }

                // Exit application
                if (command == "E")
                {
                    break;
                }
            }

        }

        private static void CommandContact(DbContext db)
        {
            for (;;)
            {
                OutputMenuContact();
                var command = GetInput().Trim().ToUpper();
                Console.WriteLine("Command: " + command);
                switch (command)
                {
                    case "L":
                        CommandContactList(db);
                        break;
                    case "C":
                        break;
                    case "R":
                        break;
                    case "U":
                        break;
                    case "D":
                        break;
                    case "E":
                        break;
                    default:
                        break;
                }
                // Exit to upper level menu
                if (command == "E")
                {
                    break;
                }

            }
        }

        private static void CommandContactList(DbContext db)
        {
            Console.WriteLine();
            Console.WriteLine("Count: " + db.Contacts.Count);
            var counter = 1;
            foreach (var dbContact in db.Contacts)
            {
                Console.WriteLine($"No {counter} - {dbContact}");
                counter++;
            }
        }

        #region menus

        private static void OutputMenu(string menuName, List<string> menuItems)
        {
            Console.WriteLine($"------- {menuName} menu -------");

            foreach (var menuItem in menuItems)
            {
                Console.WriteLine(menuItem);
            }

            Console.WriteLine("X - Go back!");
        }

        private static void OutputMenuMain()
        {
            OutputMenu("Main",
                new List<string>()
                {
                    "I - Initialize database",
                    "T - Contact type CRUD",
                    "P - Person CRUD",
                    "C - Contact CRUD",

                }
                );
        }

        private static void OutputMenuContactType()
        {
            Console.WriteLine("---- Contact Type menu ----");
            Console.WriteLine("L - List");
            Console.WriteLine("C - Create");
            Console.WriteLine("R - Read");
            Console.WriteLine("U - Update");
            Console.WriteLine("D - Delete");
            Console.WriteLine("E - Exit to main menu");
        }

        private static void OutputMenuPerson()
        {
            Console.WriteLine("---- Person menu ----");
            Console.WriteLine("L - List");
            Console.WriteLine("C - Create");
            Console.WriteLine("R - Read");
            Console.WriteLine("U - Update");
            Console.WriteLine("D - Delete");
            Console.WriteLine("E - Exit to main menu");
        }
        private static void OutputMenuContact()
        {
            Console.WriteLine("---- Contact menu ----");
            Console.WriteLine("L - List");
            Console.WriteLine("C - Create");
            Console.WriteLine("R - Read");
            Console.WriteLine("U - Update");
            Console.WriteLine("D - Delete");
            Console.WriteLine("E - Exit to main menu");
        }

        #endregion

        private static void CommandPerson(DbContext db)
        {
            for (;;)
            {
                OutputMenuPerson();
                var command = GetInput().Trim().ToUpper();
                Console.WriteLine("Command: " + command);
                switch (command)
                {
                    case "L":
                        CommandPersonList(db);
                        break;
                    case "C":
                        CommandPersonCreate(db);
                        break;
                    case "R":
                        CommandPersonRead(db);
                        break;
                    case "U":
                        break;
                    case "D":
                        CommandPersonDelete(db);
                        break;
                    case "E":
                        break;
                    default:
                        break;
                }
                // Exit to upper level menu
                if (command == "E")
                {
                    break;
                }

            }

        }

        private static void CommandPersonRead(DbContext db)
        {
            var p = db.Persons.FirstOrDefault(x => x.FirstName.ToLower() == "andres");
            if (p == null)
            {
                Console.WriteLine("No persons in db!");
                return;
            }

            Console.WriteLine(p);
            foreach (var pContact in p.Contacts)
            {
                Console.WriteLine($"    {pContact}");
            }
        }

        private static void CommandPersonDelete(DbContext db)
        {
            if (db.Persons.Count == 0)
            {
                Console.WriteLine("List is empty!");
                return;
            }

            CommandPersonList(db);
            Console.WriteLine("Give the number of record to delete (0 to cancel):");
            var command = Console.ReadLine().Trim();
            if (command != "0")
            {
                //var recordNoDoDelete = int.Parse(command) - 1;
                var recordNoDoDelete = 0;
                if (int.TryParse(command, out recordNoDoDelete))
                {
                    recordNoDoDelete = recordNoDoDelete - 1;
                    if (recordNoDoDelete < 0 || (db.Persons.Count - 1) < recordNoDoDelete)
                    {
                        Console.WriteLine("No such element in list!");
                    }
                    else
                    {
                        Console.Write("Deleting record no: " + recordNoDoDelete);
                        db.Persons.RemoveAt(recordNoDoDelete);
                        Console.WriteLine(" done!");
                    }
                }
                else
                {
                    Console.WriteLine("Please enter an number!");
                }
            }

        }

        private static void CommandPersonCreate(DbContext db)
        {
            Console.WriteLine();
            Console.Write("First name:");
            var firstName = Console.ReadLine();
            Console.Write("Last name:");
            var lastName = Console.ReadLine();

            try
            {
                db.Persons.Add(new Person()
                {
                    FirstName = firstName,
                    LastName = lastName
                });
                Console.WriteLine("Person added. Count in db: " + db.Persons.Count);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error during insertion! Message:" + ex.Message);
            }

            Console.WriteLine();
        }

        private static void CommandPersonList(DbContext db)
        {
            Console.WriteLine();
            Console.WriteLine("Count: " + db.Persons.Count);
            var counter = 1;
            foreach (var dbPerson in db.Persons)
            {
                Console.WriteLine($"No {counter} - {dbPerson}");
                counter++;
            }
        }

        private static void CommandContactType(DbContext db)
        {
            for (;;)
            {
                OutputMenuContactType();
                var command = GetInput().Trim().ToUpper();
                Console.WriteLine("Command: " + command);
                switch (command)
                {
                    case "L":
                        CommandContactTypeList(db);
                        break;
                    case "C":
                        break;
                    case "R":
                        break;
                    case "U":
                        break;
                    case "D":
                        break;
                    case "E":
                        break;
                    default:
                        break;
                }
                // Exit to upper level menu
                if (command == "E")
                {
                    break;
                }

            }
        }

        private static void CommandContactTypeList(DbContext db)
        {
            Console.WriteLine();
            Console.WriteLine("Count: " + db.ContactTypes.Count);
            foreach (var dbContactType in db.ContactTypes)
            {
                Console.WriteLine("Contact Type: " + dbContactType);
            }
        }

        static string GetInput()
        {
            Console.Write(">");
            var res = Console.ReadLine();
            return res;
        }

    }


}
