﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Domain
{
    public class Person
    {
        // this field is shared between all instances of this class
        private static int _id;

        public Person()
        {
            _id++;
            PersonId = _id;
        }

        public int PersonId { get; }

        private string _firstName;
        public string FirstName
        {
            get { return _firstName; }
            set { _firstName = FixNameString(value); }
        }

        private string _lastName;
        public string LastName
        {
            get { return _lastName; }
            set { _lastName = FixNameString(value); }
        }

        public List<Contact> Contacts { get; set; } = new List<Contact>();

        // Getters only (ie read-only properties)
        public string FirstLastName => FirstName + " " + LastName;
        public string LastFirstName => LastName + " " + FirstName;

        private string FixNameString(string name)
        {
            // remove spaces in front and back
            name = name.Trim();

            // check blacklisted
            foreach (var blackListChar in NameCharacterBlackList.BlackListChars)
            {
                if (name.Contains(blackListChar))
                {
                    throw new 
                        ArgumentException("Blacklisted char found in name ('"+blackListChar+"')!");
                }
            }

            if (name.Length < 1)
            {
                throw new ArgumentException("String is too short (minimum is 1 letter)!");
            }

            // remove multiple spaces from string
            for (;name.Contains("  ");)
            {
                name = name.Replace("  ", " ");
            }

            // Hyphens in name without surrounding spaces


            // first letter of every word is capitalized, all else - small
            // word separation - either by space or by hyphen
            // "Jüri Mari", "Jüri-Mari"
            name = name.ToLower();
            var wordArray = name.Split(' ');
            name = "";
            foreach (var singleWord in wordArray)
            {
                var singleWordStartingCap = 
                    singleWord.Substring(0, 1).ToUpper() + 
                    singleWord.Substring(1);
                name = name + " " + singleWordStartingCap;
            }
            name = name.Trim();

            return name;
        }

        public override string ToString()
        {
            return $"{PersonId} - {FirstLastName} has {Contacts.Count} contacts";
        }
    }
}
